@@ -4,45 +4,163 @@ use warnings;
 package App::Spec;
 use 5.010;
 
-our $VERSION = 'v0.15.0'; # VERSION
+our $VERSION = '0.002'; # VERSION
 
-use App::Spec::Subcommand;
+use App::Spec::Command;
 use App::Spec::Option;
 use App::Spec::Parameter;
+use List::Util qw/ any /;
+use YAML::XS ();
+use Storable qw/ dclone /;
 
 use Moo;
 
-with('App::Spec::Role::Command');
-
+has name => ( is => 'rw' );
+has appspec => ( is => 'rw' );
+has class => ( is => 'rw' );
 has title => ( is => 'rw' );
+has markup => ( is => 'rw', default => 'pod' );
+has options => ( is => 'rw' );
+has parameters => ( is => 'rw' );
+has subcommands => ( is => 'rw', default => sub { +{} } );
 has abstract => ( is => 'rw' );
+has description => ( is => 'rw' );
 
+my $DATA = do { local $/; <DATA> };
+my $default_spec;
 
-
+sub _read_default_spec {
+    $default_spec ||= YAML::XS::Load($DATA);
+    return dclone $default_spec;
+}
 
 sub runner {
-    my ($self, %args) = @_;
+    my ($self) = @_;
     my $class = $self->class;
-    my $cmd = $class->new;
-    my $run = App::Spec::Run->new({
-        spec => $self,
-        cmd => $cmd,
-        %args,
+    my $run = $class->new({
+        spec => $self
     });
     return $run;
 }
 
+sub read {
+    my ($class, $file) = @_;
+    unless (defined $file) {
+        die "No filename given";
+    }
+
+    my $spec;
+    if (ref $file eq 'GLOB') {
+        my $data = do { local $/; <$file> };
+        $spec = eval { YAML::XS::Load($data) };
+    }
+    elsif (not ref $file) {
+        $spec = eval { YAML::XS::LoadFile($file) };
+    }
+    elsif (ref $file eq 'SCALAR') {
+        my $data = $$file;
+        $spec = eval { YAML::XS::Load($data) };
+    }
+    elsif (ref $file eq 'HASH') {
+        $spec = $file;
+    }
+
+    unless ($spec) {
+        die "Error reading '$file': $@";
+    }
+
+    my $has_subcommands = $spec->{subcommands} ? 1 : 0;
+    my $default;
+    {
+        $default = $class->_read_default_spec;
+
+        for my $opt (@{ $default->{options} }) {
+            my $name = $opt->{name};
+            unless (any { $_->{name} eq $name } @{ $spec->{options} }) {
+                push @{ $spec->{options} }, $opt;
+            }
+        }
+
+        if ($has_subcommands) {
+            for my $key (keys %{ $default->{subcommands} } ) {
+                my $cmd = $default->{subcommands}->{ $key };
+                $spec->{subcommands}->{ $key } ||= $cmd;
+            }
+        }
+    }
+
+    my $commands;
+    if ($has_subcommands) {
+        # add subcommands to help command
+        my $help_subcmds = $spec->{subcommands}->{help}->{subcommands} ||= {};
+        $class->_add_subcommands($help_subcmds, $spec->{subcommands}, { subcommand_required => 0 });
+
+        for my $name (keys %{ $spec->{subcommands} || [] }) {
+            my $cmd = $spec->{subcommands}->{ $name };
+            $commands->{ $name } = App::Spec::Command->build({
+                name => $name,
+                %$cmd,
+            });
+        }
+    }
+
+    my $self = $class->new({
+        name => $spec->{name},
+        appspec => $spec->{appspec},
+        class => $spec->{class},
+        title => $spec->{title},
+        markup => $spec->{markup},
+        options => [map {
+            App::Spec::Option->build($_)
+        } @{ $spec->{options} || [] }],
+        parameters => [map {
+            App::Spec::Parameter->build($_)
+        } @{ $spec->{parameters} || [] }],
+        subcommands => $commands,
+        abstract => $spec->{abstract},
+        description => $spec->{description},
+    });
+    return $self;
+}
+
+sub _add_subcommands {
+    my ($self, $commands1, $commands2, $ref) = @_;
+    for my $name (keys %{ $commands2 || {} }) {
+        next if $name eq "help";
+        my $cmd = $commands2->{ $name };
+        $commands1->{ $name } = {
+            name => $name,
+            subcommands => {},
+            %$ref,
+        };
+        my $subcmds = $cmd->{subcommands} || {};
+        $self->_add_subcommands($commands1->{ $name }->{subcommands}, $subcmds, $ref);
+    }
+}
+
+sub has_subcommands {
+    my ($self) = @_;
+    return $self->subcommands ? 1 : 0;
+}
+
 sub usage {
     my ($self, %args) = @_;
     my $cmds = $args{commands};
     my %highlights = %{ $args{highlights} || {} };
-    my $colored = $args{colored} || sub { $_[1] };
+    my $color = $args{color};
     my $appname = $self->name;
 
+    if ($color) {
+        require Term::ANSIColor;
+    }
+
     my $abstract = $self->abstract // '';
     my $title = $self->title;
     my ($options, $parameters, $subcmds) = $self->_gather_options_parameters($cmds);
-    my $header = $colored->(['bold'], "$appname - $title");
+    my $header = "$appname - $title";
+    if ($color) {
+        $header = Term::ANSIColor::colored([qw/ bold /], $header);
+    }
     my $usage = <<"EOM";
 $header
 $abstract
@@ -50,7 +168,10 @@ $abstract
 EOM
 
     my $body = '';
-    my $usage_header = $colored->([qw/ bold /], "Usage:");
+    my $usage_header = "Usage:";
+    if ($color) {
+        $usage_header = Term::ANSIColor::colored([qw/ bold /], $usage_header);
+    }
     $usage .= "$usage_header $appname";
     $usage .= " @$cmds" if @$cmds;
     if (keys %$subcmds) {
@@ -58,37 +179,19 @@ EOM
         my @table;
         my $usage_string = "<subcommands>";
         my $header = "Subcommands:";
-        if ($highlights{subcommands}) {
-            $colored->([qw/ bold red /], $usage_string);
-            $colored->([qw/ bold red /], $header);
+        if ($color and $highlights{subcommands}) {
+            $usage_string = Term::ANSIColor::colored([qw/ bold red /], $usage_string);
+            $header = Term::ANSIColor::colored([qw/ bold red /], $header);
         }
-        else {
-            $colored->([qw/ bold /], $header);
+        elsif ($color) {
+            $header = Term::ANSIColor::colored([qw/ bold /], $header);
         }
         $usage .= " $usage_string";
         $body .= "$header\n";
-
-        my %keys;
-        @keys{ keys %$subcmds } = ();
-        my @keys;
-        if (@$cmds) {
-            @keys = sort keys %keys;
-        }
-        else {
-            for my $key (qw/ help _meta /) {
-                if (exists $keys{ $key }) {
-                    push @keys, $key;
-                    delete $keys{ $key };
-                }
-            }
-            unshift @keys, sort keys %keys;
-        }
-        for my $name (@keys) {
+        for my $name (sort keys %$subcmds) {
             my $cmd_spec = $subcmds->{ $name };
             my $summary = $cmd_spec->summary;
-            my @lines = split m/\n/, $summary;
-            push @table, [$name, $lines[0] // ''];
-            push @table, ['', $_] for map { s/^ +//; $_ } @lines[1 .. $#lines];
+            push @table, [$name, $summary];
             if (length $name > $maxlength) {
                 $maxlength = length $name;
             }
@@ -103,37 +206,35 @@ EOM
         for my $param (@$parameters) {
             my $name = $param->name;
             my $highlight = $highlights{parameters}->{ $name };
-            push @highlights, $highlight ? 1 : 0;
+            push @highlights, ($color and $highlight) ? 1 : 0;
             my $summary = $param->summary;
             my $param_usage_header = $param->to_usage_header;
-            if ($highlight) {
-                $colored->([qw/ bold red /], $param_usage_header);
+            if ($color and $highlight) {
+                $param_usage_header = Term::ANSIColor::colored([qw/ bold red /], $param_usage_header);
             }
             $usage .= " " . $param_usage_header;
             my ($req, $multi) = (' ', '  ');
             if ($param->required) {
                 $req = "*";
             }
-            if ($param->mapping) {
-                $multi = '{}';
-            }
-            elsif ($param->multiple) {
+            if ($param->multiple) {
                 $multi = '[]';
             }
 
             my $flags = $self->_param_flags_string($param);
 
-            my @lines = split m/\n/, $summary;
-            push @table, [$name, $req, $multi, ($lines[0] // '') . $flags];
-            push @table, ['', ' ', '  ', $_] for map { s/^ +//; $_ } @lines[1 .. $#lines];
+            push @table, [$name, $req, $multi, $summary . $flags];
             if (length $name > $maxlength) {
                 $maxlength = length $name;
             }
         }
-        my $parameters_string = $colored->([qw/ bold /], "Parameters:");
+        my $parameters_string = "Parameters:";
+        if ($color) {
+            $parameters_string = Term::ANSIColor::colored([qw/ bold /], $parameters_string);
+        }
         $body .= "$parameters_string\n";
         my @lines = $self->_output_table(\@table, [$maxlength]);
-        my $lines = $self->_colorize_lines(\@lines, \@highlights, $colored);
+        my $lines = $self->_colorize_lines(\@lines, \@highlights);
         $body .= $lines;
     }
 
@@ -145,7 +246,7 @@ EOM
         for my $opt (sort { $a->name cmp $b->name } @$options) {
             my $name = $opt->name;
             my $highlight = $highlights{options}->{ $name };
-            push @highlights, $highlight ? 1 : 0;
+            push @highlights, ($color and $highlight) ? 1 : 0;
             my $aliases = $opt->aliases;
             my $summary = $opt->summary;
             my @names = map {
@@ -159,23 +260,21 @@ EOM
             if ($opt->required) {
                 $req = "*";
             }
-            if ($opt->mapping) {
-                $multi = '{}';
-            }
-            elsif ($opt->multiple) {
+            if ($opt->multiple) {
                 $multi = '[]';
             }
 
             my $flags = $self->_param_flags_string($opt);
 
-            my @lines = split m/\n/, $summary;
-            push @table, [$string, $req, $multi, ($lines[0] // '') . $flags];
-            push @table, ['', ' ', '  ', $_ ] for map { s/^ +//; $_ } @lines[1 .. $#lines];
+            push @table, [$string, $req, $multi, $summary . $flags];
+        }
+        my $options_string = "Options:";
+        if ($color) {
+            $options_string = Term::ANSIColor::colored([qw/ bold /], $options_string);
         }
-        my $options_string = $colored->([qw/ bold /], "Options:");
         $body .= "\n$options_string\n";
         my @lines = $self->_output_table(\@table, [$maxlength]);
-        my $lines = $self->_colorize_lines(\@lines, \@highlights, $colored);
+        my $lines = $self->_colorize_lines(\@lines, \@highlights);
         $body .= $lines;
     }
 
@@ -191,20 +290,29 @@ sub _param_flags_string {
     if ($param->multiple) {
         push @flags, "multiple";
     }
-    if ($param->mapping) {
-        push @flags, "mapping";
-    }
     my $flags = @flags ? " (" . join("; ", @flags) . ")" : '';
     return $flags;
 }
 
+sub generate_pod {
+    my ($self) = @_;
+
+    require App::Spec::Pod;
+    my $generator = App::Spec::Pod->new(
+        spec => $self,
+    );
+    my $pod = $generator->generate;
+    return $pod;
+
+}
+
 sub _colorize_lines {
-    my ($self, $lines, $highlights, $colored) = @_;
+    my ($self, $lines, $highlights) = @_;
     my $output = '';
     for my $i (0 .. $#$lines) {
         my $line = $lines->[ $i ];
         if ($highlights->[ $i ]) {
-            $colored->([qw/ bold red /], $line);
+            $line = Term::ANSIColor::colored([qw/ bold red /], $line);
         }
         $output .= $line;
     }
@@ -252,16 +360,16 @@ sub generate_completion {
 
     if ($shell eq "zsh") {
         require App::Spec::Completion::Zsh;
-        my $completer = App::Spec::Completion::Zsh->new(
+        my $completer = App::Spec::Completion::Zsh->new({
             spec => $self,
-        );
+        });
         return $completer->generate_completion(%args);
     }
     elsif ($shell eq "bash") {
         require App::Spec::Completion::Bash;
-        my $completer = App::Spec::Completion::Bash->new(
+        my $completer = App::Spec::Completion::Bash->new({
             spec => $self,
-        );
+        });
         return $completer->generate_completion(%args);
     }
 }
@@ -281,12 +389,13 @@ sub make_getopt {
         }
         $specs->{ $name } = $opt;
         if ($opt->multiple) {
-            if ($opt->type eq 'flag') {
-                $spec .= '+';
-            }
-            elsif ($opt->mapping) {
-                $result->{ $name } = {};
-                $spec .= '%';
+            if ($opt->multiple) {
+                if ($opt->type eq 'flag') {
+                    $spec .= '+';
+                }
+                else {
+                    $spec .= '@';
+                }
             }
             else {
                 $result->{ $name } = [];
@@ -312,13 +421,9 @@ This module represents a specification of a command line tool.
 Currently it can read the spec from a YAML file or directly from a data
 structure in perl.
 
-It uses the role L<App::Spec::Role::Command>.
-
 The L<App::Spec::Run> module is the framework which will run the actual
 app.
 
-Have a look at the L<App::Spec::Tutorial> for how to write an app.
-
 In the examples directory you will find the app C<myapp> which is supposed
 to demonstrate everything that App::Spec supports right now.
 
@@ -331,25 +436,21 @@ Your script:
     $run->run;
 
     # this is equivalent to
-    #my $run = App::Spec::Run->new(
+    #my $run = Your::App->new({
     #    spec => $spec,
-    #    cmd => Your::App->new,
-    #);
+    #});
     #$run->run;
 
 Your App class:
 
     package Your::App;
-    use base 'App::Spec::Run::Cmd';
+    use base 'App::Spec::Run';
 
     sub command1 {
-        my ($self, $run) = @_;
-        my $options = $run->options;
-        my $param = $run->parameters;
+        my ($self) = @_;
+        my $options = $self->options;
+        my $param = $self->parameters;
         # Do something
-        $run->out("Hello world!");
-        $run->err("oops");
-        # you can also use print directly
     }
 
 
@@ -361,18 +462,6 @@ Your App class:
 
     my $spec = App::Spec->read("/path/to/myapp-spec.yaml");
 
-=item load_data
-
-Takes a file, hashref or glob and returns generated appspec hashref
-
-    my $hash = $class->load_data($file);
-
-=item build
-
-Builds objects out of the hashref
-
-    my $appspec = App::Spec->build(%hash);
-
 =item runner
 
 Returns an instance of the your app class
@@ -402,6 +491,10 @@ Generates shell completion script for the spec.
         shell => "zsh",
     );
 
+=item generate_pod
+
+    my $pod = $spec->generate_pod;
+
 =item make_getopt
 
 Returns options for Getopt::Long
@@ -418,12 +511,41 @@ Accessors for the things defined in the spec (file)
 
 L<App::AppSpec> - Utilities for App::Spec authors
 
-=head1 LICENSE
-
-This library is free software and may be distributed under the same terms
-as perl itself.
-
 =cut
 
 1;
 
+__DATA__
+options:
+    -   name: help
+        description: Show command help
+        type: flag
+        aliases:
+        - h
+subcommands:
+    help:
+        op: cmd_help
+        summary: Show command help
+        subcommand_required: 0
+        options:
+        -   name: all
+            type: flag
+    _complete:
+        summary: Generate self completion
+        op: cmd_self_completion
+        options:
+            -   name: name
+                description: name of the program
+            -   name: zsh
+                description: for zsh
+                type: flag
+            -   name: bash
+                description: for bash
+                type: flag
+#            -   name: without-description
+#                type: flag
+#                default: false
+#                description: generate without description
+    _pod:
+        summary: Generate self pod
+        op: cmd_self_pod
