@@ -1,9 +1,10 @@
-# ASTRACT: Generates Pod from App::Spec objects
 use strict;
 use warnings;
 package App::Spec::Pod;
 
-our $VERSION = 'v0.15.0'; # VERSION
+our $VERSION = '0.002'; # VERSION
+
+use Text::Table;
 
 use Moo;
 
@@ -69,7 +70,7 @@ sub subcommand_pod {
         @keys = sort keys %keys;
     }
     else {
-        for my $key (qw/ help _meta /) {
+        for my $key (qw/ help _complete _pod /) {
             if (exists $keys{ $key }) {
                 push @keys, $key;
                 delete $keys{ $key };
@@ -152,74 +153,43 @@ sub params2pod {
     my ($self, %args) = @_;
     my $params = $args{parameters};
     my @rows;
+    my $tb = Text::Table->new;
     for my $param (@$params) {
         my $required = $param->required ? '*' : '';
         my $summary = $param->summary;
         my $multi = '';
-        if ($param->mapping) {
-            $multi = '{}';
-        }
-        elsif ($param->multiple) {
+        if ($param->multiple) {
             $multi = '[]';
         }
         my $flags = $self->spec->_param_flags_string($param);
-        my @lines = split m/\n/, $summary;
-        push @rows, ["    " . $param->name, " " . $required, $multi, ($lines[0] // '') . $flags];
-        push @rows, ["    " , " ", '', $_] for map {s/^ +//; $_ } @lines[1 .. $#lines];
+        push @rows, ["    " . $param->name, " " . $required, $multi, $summary . $flags];
     }
-    my $test = $self->simple_table(\@rows);
-    return $test;
-}
-
-sub simple_table {
-    my ($self, $rows) = @_;
-    my @widths;
-
-    for my $row (@$rows) {
-        for my $i (0 .. $#$row) {
-            my $col = $row->[ $i ];
-            $widths[ $i ] ||= 0;
-            if ( $widths[ $i ] < length $col) {
-                $widths[ $i ] = length $col;
-            }
-        }
-    }
-    my $format = join ' ', map { "%-" . ($_ || 0) . "s" } @widths;
-    my @lines;
-    for my $row (@$rows) {
-        my $string = sprintf "$format\n", map { $_ // '' } @$row;
-        push @lines, $string;
-    }
-    return join '', @lines;
-
+    $tb->load(@rows);
+    return "$tb";
 }
 
 sub options2pod {
     my ($self, %args) = @_;
     my $options = $args{options};
     my @rows;
+    my $tb = Text::Table->new;
     for my $opt (@$options) {
         my $name = $opt->name;
         my $aliases = $opt->aliases;
         my $summary = $opt->summary;
         my $required = $opt->required ? '*' : '';
         my $multi = '';
-        if ($opt->mapping) {
-            $multi = '{}';
-        }
-        elsif ($opt->multiple) {
+        if ($opt->multiple) {
             $multi = '[]';
         }
         my @names = map {
             length $_ > 1 ? "--$_" : "-$_"
         } ($name, @$aliases);
         my $flags = $self->spec->_param_flags_string($opt);
-        my @lines = split m/\n/, $summary;
-        push @rows, ["    @names", " " . $required, $multi, ($lines[0] // '') . $flags];
-        push @rows, ["    ", " " , '', $_ ] for map {s/^ +//; $_ } @lines[1 .. $#lines];
+        push @rows, ["    @names", " " . $required, $multi, $summary . $flags];
     }
-    my $test = $self->simple_table(\@rows);
-    return $test;
+    $tb->load(@rows);
+    return "$tb";
 }
 
 sub markup {
@@ -239,69 +209,3 @@ sub swim2pod {
 }
 
 1;
-
-__END__
-
-=pod
-
-=head1 NAME
-
-App::Spec::Pod - Generates Pod from App::Spec objects
-
-=head1 SYNOPSIS
-
-    my $generator = App::Spec::Pod->new(
-        spec => $appspec,
-    );
-    my $pod = $generator->generate;
-
-=head1 METHODS
-
-=over 4
-
-=item generate
-
-    my $pod = $generator->generate;
-
-=item markup
-
-    $pod->markup(text => \$abstract);
-
-Applies markup defined in the spec to the text argument.
-
-=item options2pod
-
-    my $option_string = "Options:\n\n" . $self->options2pod(
-        options => $options,
-    );
-
-=item params2pod
-
-    my $param_string = "Parameters:\n\n" . $self->params2pod(
-        parameters => $parameters,
-    );
-
-=item subcommand_pod
-
-Generates pod for subcommands recursively
-
-    my @pod = $self->subcommand_pod(
-        previous => [@previous_subcmds],
-        commands => $subcmds,
-    );
-
-=item swim2pod
-
-    my $pod = $self->swim2pod($swim);
-
-Converts Swim markup to Pod.
-See L<Swim>.
-
-=item spec
-
-Accessor for L<App::Spec> object
-
-=back
-
-=cut
-
