@@ -3,36 +3,26 @@ use warnings;
 use strict;
 use 5.010;
 
-use Ref::Util qw/ is_arrayref is_hashref /;
-
-use base 'App::Spec::Run::Cmd';
+use base 'App::Spec::Run';
 
 sub _dump_hash {
     my ($self, $hash) = @_;
     my @strings;
     for my $key (sort keys %$hash) {
-        next unless defined $hash->{ $key };
-        my $value = $hash->{ $key };
-        if (is_hashref($value)) {
-            for my $key2 (sort keys %$value) {
-                push @strings, "$key=($key2=$value->{ $key2 })";
-            }
-        }
-        else {
-            push @strings, "$key=$value";
-        }
+    next unless defined $hash->{ $key };
+        push @strings, "$key=$hash->{ $key }";
     }
     return join ",", @strings;
 }
 
 sub cook {
-    my ($self, $run) = @_;
-    my $param = $run->parameters;
-    my $opt = $run->options;
+    my ($self) = @_;
+    my $param = $self->parameters;
+    my $opt = $self->options;
     if ($ENV{PERL5_APPSPECRUN_TEST}) {
-        $run->out("Subcommands: cook");
-        $run->out("Options: " . $self->_dump_hash($opt));
-        $run->out("Parameters: " .  $self->_dump_hash($param));
+        say "Subcommands: cook";
+        say "Options: " . $self->_dump_hash($opt);
+        say "Parameters: " .  $self->_dump_hash($param);
         return;
     }
 
@@ -44,8 +34,8 @@ sub cook {
     push @with, $with if $with;
     push @with, "sugar" if $opt->{sugar};
 
-    $run->out("Starting to cook $param->{drink}"
-    . (@with ? " with ". (join " and ", @with) : ''));
+    say "Starting to cook $param->{drink}"
+    . (@with ? " with ". (join " and ", @with) : '');
 }
 
 my %countries = (
@@ -65,16 +55,16 @@ my %countries = (
 );
 
 sub weather {
-    my ($self, $run) = @_;
-    my $param = $run->parameters;
+    my ($self) = @_;
+    my $param = $self->parameters;
     my $cities = $param->{city};
     for my $city (@$cities) {
         my $info = $countries{ $param->{country} }->{ $city };
         my $output = sprintf "Weather in %s/%s: %s", $param->{country}, $city, $info->{weather};
-        if ($run->options->{temperature}) {
+        if ($self->options->{temperature}) {
             my $temp = $info->{temperature};
             my $label = "°C";
-            if ($run->options->{fahrenheit}) {
+            if ($self->options->{fahrenheit}) {
               $temp = int($temp * 9 / 5 + 32);
               $label = "F";
             }
@@ -89,19 +79,19 @@ sub countries {
 }
 
 sub cities {
-    my ($self, $run) = @_;
-    my $country = $run->options->{country};
+    my ($self) = @_;
+    my $country = $self->{options}->{country};
     my @countries = @$country ? @$country : sort keys %countries;
     say for map { sort keys %$_ } @countries{ @countries };
 }
 
 sub weather_complete {
-    my ($self, $run, $args) = @_;
+    my ($self, $args) = @_;
     my $runmode = $args->{runmode};
     return if $runmode ne "completion";
     my $comp_param = $args->{parameter};
 
-    my $param = $run->parameters;
+    my $param = $self->parameters;
     if ($comp_param eq "city") {
         my $country = $param->{country};
         my $cities = $countries{ $country } or return;
@@ -115,15 +105,9 @@ sub weather_complete {
 }
 
 sub palindrome{
-    my ($self, $run) = @_;
-    my $string = $run->parameters->{string};
-    my $argv = $run->argv;
-    if (@$argv) {
-        $run->err("Sorry, only one palindrome at a time");
-        $run->halt(1);
-        return;
-    }
-    $run->out( ($string eq reverse $string) ? "yes" : "nope" );
+    my ($self) = @_;
+    my $string = $self->parameters->{string};
+    say +($string eq reverse $string) ? "yes" : "nope";
 }
 
 my %units = (
@@ -180,8 +164,8 @@ my %conversions = (
 );
 
 sub convert {
-    my ($self, $run) = @_;
-    my $param = $run->parameters;
+    my ($self) = @_;
+    my $param = $self->parameters;
     my $type = $param->{type};
     my $source = $param->{source};
     my $targets = $param->{target};
@@ -191,101 +175,44 @@ sub convert {
         my $sub = $conversions{ $type }->{ $key };
         my $result = $sub->($value);
         my $label = $units{ $type }->{ $target }->{label};
-        $run->out("$result$label");
+        say "$result$label";
     }
 }
 
-sub config {
-    my ($self, $run, $args) = @_;
-    my $opt = $run->options;
-    my $param = $run->parameters;
-    if ($ENV{PERL5_APPSPECRUN_TEST}) {
-        $run->out("Options: " . $self->_dump_hash($opt));
-        return;
-    }
-    warn __PACKAGE__.':'.__LINE__.$".Data::Dumper->Dump([\$opt], ['opt']);
-}
-
-sub data {
-    my ($self, $run) = @_;
-    my $opt = $run->options;
-    my $item = $opt->{item};
-
-    my $ref;
-    if ($item eq 'hash') {
-        $ref = {
-            foo => 23,
-        };
-    }
-    elsif ($item eq 'table') {
-        $ref = [
-            [qw/ FOO BAR /],
-            [ qw/ a b /],
-            [ qw/ c d /],
-        ];
-    }
-
-    # instead of print Data::Dumper->Dump($ref) or doing your own YAML or JSON
-    # encoding simply pass it to App::Spec
-    $run->out("Data '$item':");
-    $run->out($ref);
-}
 
 sub convert_complete {
-    my ($self, $run, $args) = @_;
-    my $errors = $run->validation_errors;
+    my ($self, $args) = @_;
     my $runmode = $args->{runmode};
     return if ($runmode ne "completion" and $runmode ne "validation");
     my $comp_param = $args->{parameter};
-    my $param = $run->parameters;
+    my $param = $self->parameters;
 
     if ($comp_param eq 'type') {
         return [sort keys %units];
     }
-    if (delete $errors->{parameters}->{type}) {
-        $run->err("Invalid type\n");
-        return;
-    }
-    if ($comp_param eq 'source') {
+    elsif ($comp_param eq 'source') {
         my $type = $param->{type};
         my $units = $units{ $type };
         return [map {
             +{ name => $_, description => $units->{ $_ }->{label} }
         } keys %$units];
     }
-    if (delete $errors->{parameters}->{source}) {
-        $run->err("Invalid source\n");
-        return;
-    }
-    delete $errors->{parameters}->{target};
-    if (delete $errors->{parameters}->{value}) {
-        $run->err("Invalid value\n");
-        return;
-    }
-    if ($comp_param eq 'target') {
+    elsif ($comp_param eq 'target') {
         my $type = $param->{type};
         my $source = $param->{source};
         my $value = $param->{value};
         my $units = $units{ $type };
         my @result;
-        if ($runmode eq "validation") {
-            return [sort keys %$units];
-        }
         for my $unit (sort keys %$units) {
             next if $unit eq $source;
             my $label = $units->{ $unit }->{label};
             my $key = $source . '_' . $unit;
             my $sub = $conversions{ $type }->{ $key };
             my $result = $sub->($value);
-            if ($ENV{PERL5_APPSPECRUN_TEST}) {
-                push @result, $unit;
-            }
-            else {
-                push @result, {
-                    name => $unit,
-                    description => "$result$label",
-                };
-            }
+            push @result, {
+                name => $unit,
+                description => "$result$label",
+            };
         }
         return \@result;
     }
