@@ -3,17 +3,17 @@ use warnings;
 use strict;
 use 5.010;
 
-use base 'App::Spec::Run::Cmd';
+use base 'App::Spec::Run';
 
 use App::Spec::Example::MyApp;
 
 sub execute {
-    my ($self, $run) = @_;
-    my $opt = $run->options;
-    my $param = $run->parameters;
+    my ($self) = @_;
+    my $opt = $self->options;
+    my $param = $self->parameters;
     if ($ENV{PERL5_APPSPECRUN_TEST}) {
-        $run->out("Options: " . App::Spec::Example::MyApp->_dump_hash($opt));
-        $run->out("Parameters: " .  App::Spec::Example::MyApp->_dump_hash($param));
+        say "Options: " . App::Spec::Example::MyApp->_dump_hash($opt);
+        say "Parameters: " .  App::Spec::Example::MyApp->_dump_hash($param);
         return;
     }
     warn __PACKAGE__.':'.__LINE__.$".Data::Dumper->Dump([\$opt], ['opt']);
